<?php

namespace ThirstyAffiliates\Models;

use ThirstyAffiliates\Abstracts\Abstract_Main_Plugin_Class;

use ThirstyAffiliates\Interfaces\Model_Interface;
use ThirstyAffiliates\Interfaces\Initiable_Interface;

use ThirstyAffiliates\Helpers\Plugin_Constants;
use ThirstyAffiliates\Helpers\Helper_Functions;

use ThirstyAffiliates\Models\Affiliate_Link;

/**
 * Model that houses the link fixer logic.
 *
 * @since 3.0.0
 */
class Link_Fixer implements Model_Interface , Initiable_Interface {

    /*
    |--------------------------------------------------------------------------
    | Class Properties
    |--------------------------------------------------------------------------
    */

    /**
     * Property that holds the single main instance of Bootstrap.
     *
     * @since 3.0.0
     * @access private
     * @var Link_Fixer
     */
    private static $_instance;

    /**
     * Model that houses the main plugin object.
     *
     * @since 3.0.0
     * @access private
     * @var Abstract_Main_Plugin_Class
     */
    private $_main_plugin;

    /**
     * Model that houses all the plugin constants.
     *
     * @since 3.0.0
     * @access private
     * @var Plugin_Constants
     */
    private $_constants;

    /**
     * Property that houses all the helper functions of the plugin.
     *
     * @since 3.0.0
     * @access private
     * @var Helper_Functions
     */
    private $_helper_functions;




    /*
    |--------------------------------------------------------------------------
    | Class Methods
    |--------------------------------------------------------------------------
    */

    /**
     * Class constructor.
     *
     * @since 3.0.0
     * @access public
     *
     * @param Abstract_Main_Plugin_Class $main_plugin      Main plugin object.
     * @param Plugin_Constants           $constants        Plugin constants object.
     * @param Helper_Functions           $helper_functions Helper functions object.
     */
    public function __construct( Abstract_Main_Plugin_Class $main_plugin , Plugin_Constants $constants , Helper_Functions $helper_functions ) {

        $this->_constants        = $constants;
        $this->_helper_functions = $helper_functions;

        $main_plugin->add_to_all_plugin_models( $this );
        $main_plugin->add_to_public_models( $this );

    }

    /**
     * Ensure that only one instance of this class is loaded or can be loaded ( Singleton Pattern ).
     *
     * @since 3.0.0
     * @access public
     *
     * @param Abstract_Main_Plugin_Class $main_plugin      Main plugin object.
     * @param Plugin_Constants           $constants        Plugin constants object.
     * @param Helper_Functions           $helper_functions Helper functions object.
     * @return Link_Fixer
     */
    public static function get_instance( Abstract_Main_Plugin_Class $main_plugin , Plugin_Constants $constants , Helper_Functions $helper_functions ) {

        if ( !self::$_instance instanceof self )
            self::$_instance = new self( $main_plugin , $constants , $helper_functions );

        return self::$_instance;

    }

    /**
     * Get data of links to be fixed.
     *
     * @since 3.0.0
     * @access public
     *
     * @param array $links   List of affiliate links to fix.
     * @param int   $post_id ID of the post currently being viewed.
     * @param array $data    Affiliate Links data.
     * @return array Affiliate Links data.
     */
    public function get_link_fixer_data( $links , $post_id = 0 , $data = array() ) {

        if ( empty( $links ) )
            return $data;

        foreach( $links as $link ) {

            $href    = esc_url_raw( $link[ 'href' ] );
            $class   = sanitize_text_field( $link[ 'class' ] );
            $key     = (int) sanitize_text_field( $link[ 'key' ] );
            $content = intval( $link[ 'is_image' ] ) ? '{image_placeholder}' : $link[ 'content' ];
            $link_id = url_to_postid( $href );

            $thirstylink = new Affiliate_Link( $link_id );

            // if ( $thirstylink->get_prop( 'status' ) != 'publish' )
            //     continue;

            $class     .= ( get_option( 'ta_disable_thirsty_link_class' ) !== "yes" && strpos( $class , 'thirstylink' ) === false ) ? ' thirstylink' : '';
            $nofollow   = $thirstylink->get_prop( 'no_follow' ) == 'global' ? get_option( 'ta_no_follow' ) : $thirstylink->get_prop( 'no_follow' );
            $new_window = $thirstylink->get_prop( 'new_window' ) == 'global' ? get_option( 'ta_new_window' ) : $thirstylink->get_prop( 'new_window' );
            $href       = ( $this->_helper_functions->is_uncloak_link( $thirstylink ) ) ? $thirstylink->get_prop( 'destination_url' ) : $thirstylink->get_prop( 'permalink' );
            $rel        = $nofollow == "yes" ? 'nofollow' : '';
            $rel       .= ' ' . $thirstylink->get_prop( 'rel_tags' );
            $target     = $new_window == "yes" ? '_blank' : '';
            $title      = ( get_option( 'ta_disable_title_attribute' ) != 'yes' ) ? 'title="' . esc_attr( str_replace( '"' , '' , $thirstylink->get_prop( 'name' ) ) ) . '" ' : '';
            $other_atts = apply_filters( 'ta_link_insert_extend_data_attributes' , array() , $thirstylink , $post_id );
            $other_atts_string = '';

            if ( is_array( $other_atts ) && ! empty( $other_atts ) ) {

                foreach ( $other_atts as $att => $att_value )
                    $other_atts_string .= $att . '="' . esc_attr( $att_value ) . '" ';
            }

            $html = '<a class="' . esc_attr( $class ) . '" ' . $title .
                    'href="' . esc_url( $href ) . '" ' .
                    'rel="' . esc_attr( trim( $rel ) ) . '" ' .
                    'target="' . esc_attr( $target ) . '" ' .
                    'data-linkid="' . esc_attr( $link_id ) . '" ' . $other_atts_string . '>' . $content . '</a>';

            $data[] = array(
                'key'      => $key,
                'html'     => $html,
                'link_id'  => $link_id,
                'is_image' => intval( $link[ 'is_image' ] )
            );
        }

        return $data;
    }

    /**
     * Ajax link fixer.
     *
     * @since 3.0.0
     * @access public
     */
    public function ajax_link_fixer() {

        if ( ! defined( 'DOING_AJAX' ) || ! DOING_AJAX )
            $response = array( 'status' => 'fail' , 'error_msg' => __( 'Invalid AJAX call' , 'thirstyaffiliates' ) );
        elseif ( ! isset( $_POST[ 'hrefs' ] ) || empty( $_POST[ 'hrefs' ] ) )
            $response = array( 'status' => 'fail' , 'error_msg' => __( 'Invalid AJAX call' , 'thirstyaffiliates' ) );
        else {

            $links    = $_POST[ 'hrefs' ];
            $post_id  = isset( $_POST[ 'post_id' ] ) ? intval( $_POST[ 'post_id' ] ) : 0;
            $response = array(
                'status' => 'success',
                'data' => $this->get_link_fixer_data( $links , $post_id )
            );
        }

        @header( 'Content-Type: application/json; charset=' . get_option( 'blog_charset' ) );
        echo wp_json_encode( $response );
        wp_die();
    }




    /*
    |--------------------------------------------------------------------------
    | Fulfill implemented interface contracts
    |--------------------------------------------------------------------------
    */

    /**
     * Execute codes that needs to run on plugin initialization.
     *
     * @since 3.0.0
     * @access public
     * @implements ThirstyAffiliates\Interfaces\Initiable_Interface
     */
    public function initialize() {

        add_action( 'wp_ajax_ta_link_fixer' , array( $this , 'ajax_link_fixer' ) );
        add_action( 'wp_ajax_nopriv_ta_link_fixer' , array( $this , 'ajax_link_fixer' ) );
    }

    /**
     * Execute link picker.
     *
     * @since 3.0.0
     * @access public
     */
    public function run() {
    }
}
